@@ -1,37 +1,56 @@
 ### =========================================================================
-### The "BSgenome" class
+### BSgenome objects
 ### -------------------------------------------------------------------------
 
+
 setClass("BSgenome",
     contains="GenomeDescription",
     representation(
-        ## source_url: permanent URL to the place where the FASTA files used
-        ## to produce the sequences below can be found (and downloaded)
+        ## Name of the BSgenome data package where the BSgenome object is
+        ## defined.
+        pkgname="character",
+
+        ## On-disk "single" and "multiple" sequences.
+        single_sequences="OnDiskNamedSequences",
+        multiple_sequences="RdaCollection",
+
+        ## Permanent URL to the place where the FASTA files that were used
+        ## to produce the "single" and "multiple" sequences above can be
+        ## found (and downloaded).
         source_url="character",
 
-        ## named vector representing the translation table from the original
+        ## Named vector representing the translation table from the original
         ## seqnames (as stored in self@seqinfo@seqnames, the 'seqinfo' slot
         ## being inherited from the GenomeDescription class) to the user
-        ## seqnames
+        ## seqnames.
         user_seqnames="character",
 
-        ## mseqnames: names of "multiple" sequences (e.g. upstream)
-        mseqnames="character",
-
-        ## where to find the serialized objects containing the sequences
-        seqs_pkgname="character",
-        seqs_dirpath="character",
-
-        ## where to find the serialized objects containing the masks
-        nmask_per_seq="integer",
-        masks_pkgname="character",
-        masks_dirpath="character",
-
-        ## for SNPs injection
+        ## For SNPs injection.
         injectSNPs_handler="InjectSNPsHandler",
 
+        ## Used for caching the single and multiple on-disk sequences.
         .seqs_cache="environment",
-        .link_counts="environment"
+        .link_counts="environment",
+
+        ## TODO: Remove the 2 slots below once all the BSgenome data packages
+        ## with masks have been split in 2 packages (naked BSgenome +
+        ## MaskedBSgenome).
+        nmask_per_seq="integer",
+        masks="RdaCollection"
+    )
+)
+
+setClass("MaskedBSgenome",
+    contains="BSgenome",
+    representation(
+        ## Name of the BSgenome data package where the MaskedBSgenome object
+        ## is defined.
+        masks_pkgname="character",
+
+        ## Only the single sequences can be masked. They should all have the
+        ## same set of masks.
+        nmask_per_seq="integer",
+        masks="RdaCollection"
     )
 )
 
@@ -41,44 +60,6 @@ setClass("BSgenome",
 ### sequences.
 ###
 
-.getObjFilepath <- function(objname, objdir)
-{
-    ## Should never happen.
-    if (objdir == "")
-        ## TODO: Put this kind of checking in a validity method for BSgenome
-        ## objects (that's what validity methods are for).
-        stop("internal anomaly: objdir is \"\"")
-    filename <- paste(objname, ".rda", sep="")
-    filepath <- file.path(objdir, filename)
-    if (!file.exists(filepath))
-        stop("file '", filepath, "' doesn't exist")
-    filepath
-}
-
-.loadSingleObject <- function(objname, objdir, objpkgname)
-{
-    filepath <- .getObjFilepath(objname, objdir)
-    tmp_env <- new.env(parent=emptyenv())
-    loaded_names <- load(filepath, envir=tmp_env)
-    ## Check that we get only 1 object
-    if (length(loaded_names) != 1L)
-        stop("file '", filepath, "' contains 0 or more than 1 serialized object. ",
-             "May be the ", objpkgname, " package is corrupted?")
-    ## ... and that it has the expected name
-    if (loaded_names != objname)
-        stop("the serialized object in file '", filepath, "' ",
-             "doesn't have the expected name. ",
-             "May be the ", objpkgname, " package is corrupted?")
-    ans <- get(objname, envir=tmp_env)
-    ## TODO: This is temporary code to make temporarily broken BSgenome
-    ## data packages work despite the big internal renaming I made in
-    ## IRanges >= 1.3.76. Remove it after all the BSgenome data packages
-    ## have been reforged.
-    if (is(ans, "XString") || is(ans, "XStringSet"))
-        ans <- updateObject(ans)
-    return(ans)
-}
-
 ### Return a new link to a cached object.
 ### 'objname' is the name of the cached object.
 ### 'cache' is the caching environment.
@@ -178,7 +159,13 @@ setMethod("SNPlocs", "BSgenome",
 
 setGeneric("mseqnames", function(x) standardGeneric("mseqnames"))
 setMethod("mseqnames", "BSgenome",
-    function(x) { if (length(x@mseqnames) == 0L) NULL else x@mseqnames }
+    function(x)
+    {
+        ans <- names(x@multiple_sequences)
+        if (length(ans) == 0L)
+            ans <- NULL
+        ans
+    }
 )
 
 setMethod("names", "BSgenome", function(x) c(seqnames(x), mseqnames(x)))
@@ -262,58 +249,97 @@ setReplaceMethod("seqnames", "BSgenome",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Constructor-like functions and generics
+### BSgenome constructor
 ###
 
-.makeSeqinfo <- function(seqnames, circ_seqs, seqs_pkgname, seqs_dirpath,
-                         provider_version)
+### 'seqnames' only used for sanity check.
+.makeBSgenomeSeqinfo <- function(single_sequences,
+                                 circ_seqs, provider_version,
+                                 seqnames)
 {
-    objname <- "seqlengths"
-    seqlengths <- .loadSingleObject(objname, seqs_dirpath, seqs_pkgname)
-    if (!identical(names(seqlengths), seqnames)) {
-        filepath <- .getObjFilepath(objname, seqs_dirpath)
-        stop("sequence names found in file '", filepath, "' are not ",
-             "identical to 'seqnames'. ",
-             "May be the ", seqs_pkgname, " package is corrupted?")
-    }
-    if (identical(circ_seqs, NA))
+    seqlengths <- seqlengths(single_sequences)
+    if (!identical(names(seqlengths), seqnames))
+        stop("sequence names found in file '",
+             seqlengthsFilepath(single_sequences),
+             "' are not identical to 'seqnames'. ",
+             "May be the data on disk is corrupted?")
+    if (identical(circ_seqs, NA)) {
         is_circ <- NA
-    else
+    } else {
         is_circ <- seqnames %in% circ_seqs
-    Seqinfo(seqnames=seqnames, seqlengths=seqlengths, isCircular=is_circ,
+    }
+    Seqinfo(seqnames=seqnames,
+            seqlengths=seqlengths,
+            isCircular=is_circ,
             genome=provider_version)
 }
 
+### NOTE: In BioC 2.14, the 'seqs_pkgname' and 'masks_pkgname' BSgenome slots
+### have been replaced with the 'pkgname' slot but the 2 corresponding
+### arguments have been kept for backward compatibility.
 BSgenome <- function(organism, species, provider, provider_version,
                      release_date, release_name, source_url,
                      seqnames, circ_seqs=NA, mseqnames,
                      seqs_pkgname, seqs_dirpath,
-                     nmask_per_seq, masks_pkgname, masks_dirpath)
+                     nmask_per_seq=0, masks_pkgname=NA, masks_dirpath=NA)
 {
-    seqinfo <- .makeSeqinfo(seqnames, circ_seqs, seqs_pkgname, seqs_dirpath,
-                            provider_version)
-    user_seqnames <- seqnames(seqinfo)
-    names(user_seqnames) <- user_seqnames
+    fa_filename <- "single_sequences.fa"
+    fa_filepath <- file.path(seqs_dirpath, fa_filename)
+    farz_filename <- paste0(fa_filename, ".rz")
+    farz_filepath <- file.path(seqs_dirpath, farz_filename)
+    if (file.exists(farz_filepath)) {
+        single_sequences <- FastaNamedSequences(farz_filepath)
+    } else if (file.exists(fa_filepath)) {
+        single_sequences <- FastaNamedSequences(fa_filepath)
+    } else {
+        single_sequences <- RdaNamedSequences(seqs_dirpath, seqnames)
+    }
+    seqinfo <- .makeBSgenomeSeqinfo(single_sequences,
+                                    circ_seqs, provider_version,
+                                    seqnames)
+    genome_description <- GenomeDescription(organism, species,
+                                            provider, provider_version,
+                                            release_date, release_name,
+                                            seqinfo)
     if (is.null(mseqnames))
         mseqnames <- character(0)
-    new("BSgenome",
-        GenomeDescription(organism, species,
-                          provider, provider_version,
-                          release_date, release_name,
-                          seqinfo),
+    multiple_sequences <- RdaCollection(seqs_dirpath, mseqnames)
+    user_seqnames <- seqnames(seqinfo)
+    names(user_seqnames) <- user_seqnames
+    nmask_per_seq <- as.integer(nmask_per_seq)
+    masks_dirpath <- as.character(masks_dirpath)
+    if (nmask_per_seq == 0L || length(seqnames) == 0L) {
+        masks <- RdaCollection(masks_dirpath, character(0))
+    } else {
+        masks <- RdaCollection(masks_dirpath, paste0(seqnames, ".masks"))
+    }
+
+    new("BSgenome", genome_description,
+        pkgname=seqs_pkgname,
+        single_sequences=single_sequences,
+        multiple_sequences=multiple_sequences,
         source_url=source_url,
         user_seqnames=user_seqnames,
-        mseqnames=mseqnames,
-        seqs_pkgname=seqs_pkgname,
-        seqs_dirpath=seqs_dirpath,
-        nmask_per_seq=as.integer(nmask_per_seq),
-        masks_pkgname=masks_pkgname,
-        masks_dirpath=masks_dirpath,
         .seqs_cache=new.env(parent=emptyenv()),
-        .link_counts=new.env(parent=emptyenv())
+        .link_counts=new.env(parent=emptyenv()),
+        nmask_per_seq=nmask_per_seq,
+        masks=masks
     )
 }
 
+MaskedBSgenome <- function(ref_bsgenome,
+                           masks_pkgname, nmask_per_seq, masks_dirpath)
+{
+    masks <- RdaCollection(masks_dirpath,
+                           paste0(seqnames(ref_bsgenome), ".masks"))
+    new("MaskedBSgenome", ref_bsgenome,
+        .seqs_cache=new.env(parent=emptyenv()),
+        .link_counts=new.env(parent=emptyenv()),
+        masks_pkgname=masks_pkgname,
+        nmask_per_seq=as.integer(nmask_per_seq),
+        masks=masks) 
+}
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### The 'show' method
@@ -375,68 +401,92 @@ setMethod("show", "BSgenome",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Subsetting.
+### List-element extraction (with [[).
 ###
 
-.loadBSgenomeSequence <- function(objname, bsgenome, user_seqname)
+.loadBSgenomeSequence <- function(x, seqname, user_seqname)
 {
-    seqs_pkgname <- bsgenome@seqs_pkgname
-    seqs_dirpath <- bsgenome@seqs_dirpath
-    nmask_per_seq <- length(masknames(bsgenome))
-    masks_pkgname <- bsgenome@masks_pkgname
-    masks_dirpath <- bsgenome@masks_dirpath
-    ans <- .loadSingleObject(objname, seqs_dirpath, seqs_pkgname)
-    if (!is(ans, "XString"))
+    idx <- match(user_seqname, x@user_seqnames)
+    if (is.na(idx)) {  # multiple sequence
+        if (substr(seqname, 1L, 8L) == "upstream") {
+            msg <- c(
+                "  Starting with BioC 2.14, upstream sequences ",
+                "are deprecated.\n",
+                "  However they can easily be extracted ",
+                "from the full genome\n  sequences with something like ",
+                "(for example for hg19):\n\n",
+                "      library(TxDb.Hsapiens.UCSC.hg19.knownGene)\n",
+                "      txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene\n",
+                "      gn <- sort(genes(txdb))\n",
+                "      up1000 <- flank(gn, width=1000)\n",
+                "      library(BSgenome.Hsapiens.UCSC.hg19)\n",
+                "      genome <- BSgenome.Hsapiens.UCSC.hg19\n",
+                "      up1000seqs <- getSeq(genome, up1000)\n\n",
+                "  IMPORTANT: Make sure you use a TxDb package (or ",
+                "TranscriptDb object)\n  that contains a gene model ",
+                "based on the exact same reference genome\n",
+                "  as the BSgenome object you pass to getSeq(). Note that ",
+                "you can make\n  your own custom TranscriptDb object from ",
+                "various annotation resources.\n",
+                "  See the makeTranscriptDbFromUCSC(), ",
+                "makeTranscriptDbFromBiomart(), and\n",
+                "  makeTranscriptDbFromGFF() functions in the ",
+                "GenomicFeatures package."
+            )
+            .Deprecated(msg=paste0(msg, collapse=""))
+        }
+        ans <- x@multiple_sequences[[seqname]]
         return(ans)
+    }
+    # single sequence
+    ans <- x@single_sequences[[seqname]]
     ## Check the length of the sequence
-    if (length(ans) != seqlengths(bsgenome)[[user_seqname]]) {
-        seq_filepath <- .getObjFilepath(objname, seqs_dirpath)
-        stop("sequence found in file '", seq_filepath, "' does ",
-             "not have the length reported by seqlengths(). ",
-             "May be the ", seqs_pkgname, " package is corrupted?")
+    if (length(ans) != seqlengths(x)[[user_seqname]]) {
+        stop(user_seqname, " sequence does not have the expected length. ",
+             "May be the data on disk is corrupted?")
     }
     ## Inject the SNPs, if any
-    snps <- SNPlocs(bsgenome, objname)
-    if (!is.null(snps)) 
+    snps <- SNPlocs(x, seqname)
+    if (!is.null(snps))
         .inplaceReplaceLetterAt(ans, snps$loc, snps$alleles_as_ambig)
     ## Load and set the built-in masks, if any
+    nmask_per_seq <- length(masknames(x))
     if (nmask_per_seq > 0L) {
-        masks_objname <- paste(objname, ".masks", sep="")
-        builtinmasks <- .loadSingleObject(masks_objname, masks_dirpath,
-                                          masks_pkgname)
+        masks_objname <- paste0(seqname, ".masks")
+        builtinmasks <- x@masks[[masks_objname]]
         if (length(builtinmasks) < nmask_per_seq) {
-            masks_filepath <- .getObjFilepath(masks_objname, masks_dirpath)
+            masks_filepath <- rdaPath(x@masks, masks_objname)
             stop("expecting ", nmask_per_seq, " built-in masks per ",
                  "single sequence, found only ", length(builtinmasks),
                  " in file '", masks_filepath, "'. ",
-                 "May be the ", masks_pkgname, " package is corrupted?")
+                 "May be the data on disk is corrupted?")
         }
         if (length(builtinmasks) > nmask_per_seq)
             builtinmasks <- builtinmasks[seq_len(nmask_per_seq)]
-        if (!identical(names(builtinmasks), masknames(bsgenome))) {
-            masks_filepath <- .getObjFilepath(masks_objname, masks_dirpath)
+        if (!identical(names(builtinmasks), masknames(x))) {
+            masks_filepath <- rdaPath(x@masks, masks_objname)
             stop("mask names found in file '", masks_filepath, "' are not ",
                  "identical to the names returned by masknames(). ",
-                 "May be the ", masks_pkgname, " package is corrupted?")
+                 "May be the data on disk is corrupted?")
         }
         masks(ans) <- builtinmasks
     }
     ans
 }
 
-.getBSgenomeSequence <- function(objname, bsgenome, user_seqname)
+.getBSgenomeSequence <- function(x, seqname, user_seqname)
 {
-    seqs_cache <- bsgenome@.seqs_cache
+    seqs_cache <- x@.seqs_cache
     ## Using the 'if (!exists()) assign(); get()' approach is NOT 100%
     ## reliable:
     ##
-    ##   if (!exists(objname, envir=seqs_cache, inherits=FALSE)) {
+    ##   if (!exists(seqname, envir=seqs_cache, inherits=FALSE)) {
     ##       ...
-    ##       assign(objname, ans, envir=seqs_cache)
+    ##       assign(seqname, ans, envir=seqs_cache)
     ##   }
-    ##   get(objname, envir=seqs_cache, inherits=FALSE)
+    ##   get(seqname, envir=seqs_cache, inherits=FALSE)
     ##
-    ## because the symbol (objname) can disappear from the cache between
+    ## because the symbol (seqname) can disappear from the cache between
     ## the moment we test for its presence and the moment we try to get it.
     ## It's not me being paranoid, we've seen this happen! One possible
     ## explanation for this is that the symbol was candidate for removal
@@ -447,17 +497,17 @@ setMethod("show", "BSgenome",
     ## garbbage collection and that in turn would trigger the removal of
     ## the symbol *before* get() had a chance to get to it. So now we use
     ## the 'try(get(...))' approach, which hopefully is 100% reliable!
-    ans <- try(get(objname, envir=seqs_cache, inherits=FALSE), silent=TRUE)
+    ans <- try(get(seqname, envir=seqs_cache, inherits=FALSE), silent=TRUE)
     if (is(ans, "try-error")) {
-        ans <- .loadBSgenomeSequence(objname, bsgenome, user_seqname)
+        ans <- .loadBSgenomeSequence(x, seqname, user_seqname)
         if (getOption("verbose"))
-            cat("caching ", objname, "\n", sep="")
-        assign(objname, ans, envir=seqs_cache)
+            cat("caching ", seqname, "\n", sep="")
+        assign(seqname, ans, envir=seqs_cache)
     }
     .linkToCachedObject(ans) <- .newLinkToCachedObject(
-                                    objname,
+                                    seqname,
                                     seqs_cache,
-                                    bsgenome@.link_counts)
+                                    x@.link_counts)
     ans
 }
 
@@ -484,8 +534,8 @@ setMethod("[[", "BSgenome",
         if (length(i) > 1)
             stop("attempt to select more than one element")
         if (is.character(i)) {
-            name <- try(match.arg(i, names(x)), silent=TRUE)
-            if (is(name, "try-error"))
+            user_seqname <- try(match.arg(i, names(x)), silent=TRUE)
+            if (is(user_seqname, "try-error"))
                 stop("no such sequence")
         } else {
             if (!is.numeric(i) || is.na(i))
@@ -493,16 +543,16 @@ setMethod("[[", "BSgenome",
             i <- as.integer(i)
             if (i < 1L || length(x) < i)
                 stop("no such sequence")
-            name <- names(x)[i]
+            user_seqname <- names(x)[i]
         }
         ## Translate user-supplied sequence named back to original name.
-        idx <- match(name, x@user_seqnames)
+        idx <- match(user_seqname, x@user_seqnames)
         if (is.na(idx)) {
-            objname <- name
+            seqname <- user_seqname  # multiple sequence
         } else {
-            objname <- names(x@user_seqnames)[[idx]]
+            seqname <- names(x@user_seqnames)[[idx]]  # single sequence
         }
-        .getBSgenomeSequence(objname, x, name)
+        .getBSgenomeSequence(x, seqname, user_seqname)
     }
 )
 
@@ -516,3 +566,4 @@ setReplaceMethod("[[", "BSgenome",
 setMethod("$", "BSgenome",
     function(x, name) x[[name]]
 )
+
