@@ -106,8 +106,8 @@ forgeSeqlengthsFile <- function(seqnames, prefix="", suffix=".fa",
 ### The "forgeSeqFiles" function.
 ###
 
-.forgeSeqFile <- function(name, prefix, suffix, seqs_srcdir, seqs_destdir,
-                          is.single.seq=TRUE, verbose=TRUE)
+.forgeRdaSeqFile <- function(name, prefix, suffix, seqs_srcdir, seqs_destdir,
+                             is.single.seq=TRUE, verbose=TRUE)
 {
     if (!isSingleString(name))
         stop("'name' must be a single string")
@@ -129,8 +129,65 @@ forgeSeqlengthsFile <- function(seqnames, prefix="", suffix=".fa",
     .saveObject(seq, name, destdir=seqs_destdir, verbose=verbose)
 }
 
+.forgeFastaRzFile <- function(seqnames, prefix, suffix,
+                              seqs_srcdir, seqs_destdir,
+                              mode, verbose=TRUE)
+{
+    if (!is.character(seqnames))
+        stop("'seqnames' must be a character vector")
+    dest_filename <- "single_sequences.fa"
+    dest_filepath <- file.path(seqs_destdir, dest_filename)
+    for (seqname in seqnames) {
+        srcpath <- getSeqSrcpaths(seqname, prefix=prefix, suffix=suffix,
+                                  seqs_srcdir=seqs_srcdir)
+        if (verbose)
+            cat("Loading '", seqname, "' sequence from FASTA file '", srcpath, "' ... ", sep="")
+        seq <- readDNAStringSet(srcpath, "fasta")
+        if (verbose)
+            cat("DONE\n")
+        if (length(seq) == 0L)
+            stop("file contains no DNA sequence")
+        if (length(seq) > 1L) {
+            warning("file contains ", length(seq), " sequences, ",
+                    "using the first sequence only")
+            seq <- seq[1L]
+        }
+        if (verbose)
+            cat("Appending '", seqname, "' sequence to FASTA file '", dest_filepath, "' ... ", sep="")
+        names(seq) <- seqname
+        writeXStringSet(seq, dest_filepath, append=TRUE, format="fasta", width=50L)
+        if (verbose)
+            cat("DONE\n")
+    }
+
+    if (mode == "fa") {
+        ## "fa" mode
+        if (verbose)
+            cat("Indexing FASTA file '", dest_filepath, "' ... ", sep="")
+        indexFa(dest_filepath)
+        if (verbose)
+            cat("DONE\n")
+    } else {
+        ## "fa.rz" mode
+        if (verbose)
+            cat("Compressing FASTA file '", dest_filepath, "' ... ", sep="")
+        farz_filepath <- sprintf("%s.rz", dest_filepath)
+        razip(dest_filepath, dest=farz_filepath)
+        unlink(dest_filepath)
+        if (verbose)
+            cat("DONE\n")
+        if (verbose)
+            cat("Indexing compressed FASTA file '", farz_filepath,
+                "' ... ", sep="")
+        indexFa(farz_filepath)
+        if (verbose)
+            cat("DONE\n")
+    }
+}
+
 forgeSeqFiles <- function(seqnames, mseqnames=NULL, prefix="", suffix=".fa",
-                          seqs_srcdir=".", seqs_destdir=".", verbose=TRUE)
+                          seqs_srcdir=".", seqs_destdir=".",
+                          mode=c("rda", "fa", "fa.rz"), verbose=TRUE)
 {
     if (length(seqnames) == 0) {
         warning("'seqnames' is empty")
@@ -148,13 +205,19 @@ forgeSeqFiles <- function(seqnames, mseqnames=NULL, prefix="", suffix=".fa",
     }
     if (!isSingleString(seqs_destdir))
         stop("'seqs_destdir' must be a single string")
-    for (name in seqnames) {
-        .forgeSeqFile(name, prefix, suffix, seqs_srcdir, seqs_destdir,
-                      is.single.seq=TRUE, verbose=verbose)
+    mode <- match.arg(mode)
+    if (mode == "rda") {  # "rda" mode
+        for (name in seqnames) {
+            .forgeRdaSeqFile(name, prefix, suffix, seqs_srcdir, seqs_destdir,
+                             is.single.seq=TRUE, verbose=verbose)
+        }
+    } else {  # "fa" and "fa.rz" modes
+        .forgeFastaRzFile(seqnames, prefix, suffix, seqs_srcdir, seqs_destdir,
+                          mode, verbose=verbose)
     }
     for (name in mseqnames) {
-        .forgeSeqFile(name, prefix, suffix, seqs_srcdir, seqs_destdir,
-                      is.single.seq=FALSE, verbose=verbose)
+        .forgeRdaSeqFile(name, prefix, suffix, seqs_srcdir, seqs_destdir,
+                         is.single.seq=FALSE, verbose=verbose)
     }
 }
 
@@ -270,7 +333,8 @@ forgeSeqFiles <- function(seqnames, mseqnames=NULL, prefix="", suffix=".fa",
 }
 
 .forgeMasksFile <- function(seqname, nmask_per_seq,
-                            seqs_destdir=".", masks_srcdir=".", masks_destdir=".",
+                            seqs_destdir=".", mode=c("rda", "fa", "fa.rz"),
+                            masks_srcdir=".", masks_destdir=".",
                             AGAPSfiles_type="gap", AGAPSfiles_name=NA,
                             AGAPSfiles_prefix="", AGAPSfiles_suffix="_gap.txt",
                             RMfiles_name=NA, RMfiles_prefix="", RMfiles_suffix=".fa.out",
@@ -292,10 +356,25 @@ forgeSeqFiles <- function(seqnames, mseqnames=NULL, prefix="", suffix=".fa",
     if (!isSingleString(masks_destdir))
         stop("'masks_destdir' must be a single string")
 
-    ## Get the length of the sequence.
-    seqfile <- file.path(seqs_destdir, paste(seqname, ".rda", sep=""))
-    load(seqfile)
-    seq <- get(seqname)
+    ## Load the sequence.
+    mode <- match.arg(mode)
+    if (mode == "rda") {  # "rda" mode
+        seqfile <- file.path(seqs_destdir, paste(seqname, ".rda", sep=""))
+        load(seqfile)
+        seq <- get(seqname)
+        remove(list=seqname)
+    } else {  # "fa" and "fa.rz" modes
+        fa_filename <- "single_sequences.fa"
+        if (mode == "fa.rz")
+            fa_filename <- paste0(fa_filename, ".rz")
+        fa_filepath <- file.path(seqs_destdir, fa_filename)
+        fafile <- FaFile(fa_filepath)
+        ## TODO: Implement "[[" method for FaFile objects (in Rsamtools) and
+        ## use it here (i.e. do 'seq <- fafile[[seqname]]' instead of the 2
+        ## lines below).
+        param <- GRanges(seqname, IRanges(1L, seqlengths(fafile)[[seqname]]))
+        seq <- scanFa(fafile, param=param)[[1L]]
+    }
     mask_width <- length(seq)
 
     ## Start with an empty mask collection (i.e. a MaskCollection of
@@ -311,7 +390,6 @@ forgeSeqFiles <- function(seqnames, mseqnames=NULL, prefix="", suffix=".fa",
         AMBmask <- .forge.AMBmask(seq, AGAPSmask)
         masks <- append(masks, AMBmask)
     }
-    remove(seq, list=seqname)
     if (nmask_per_seq >= 3) {
         RMmask <- .forge.RMmask(seqname, mask_width, masks_srcdir,
                                 RMfiles_name, RMfiles_prefix, RMfiles_suffix)
@@ -327,7 +405,8 @@ forgeSeqFiles <- function(seqnames, mseqnames=NULL, prefix="", suffix=".fa",
 }
 
 forgeMasksFiles <- function(seqnames, nmask_per_seq,
-                            seqs_destdir=".", masks_srcdir=".", masks_destdir=".",
+                            seqs_destdir=".", mode=c("rda", "fa", "fa.rz"),
+                            masks_srcdir=".", masks_destdir=".",
                             AGAPSfiles_type="gap", AGAPSfiles_name=NA,
                             AGAPSfiles_prefix="", AGAPSfiles_suffix="_gap.txt",
                             RMfiles_name=NA, RMfiles_prefix="", RMfiles_suffix=".fa.out",
@@ -338,7 +417,7 @@ forgeMasksFiles <- function(seqnames, nmask_per_seq,
         warning("'seqnames' is empty")
     for (seqname in seqnames) {
         .forgeMasksFile(seqname, nmask_per_seq,
-                        seqs_destdir=seqs_destdir,
+                        seqs_destdir=seqs_destdir, mode=mode,
                         masks_srcdir=masks_srcdir, masks_destdir=masks_destdir,
                         AGAPSfiles_type=AGAPSfiles_type, AGAPSfiles_name=AGAPSfiles_name,
                         AGAPSfiles_prefix=AGAPSfiles_prefix, AGAPSfiles_suffix=AGAPSfiles_suffix,
@@ -350,11 +429,11 @@ forgeMasksFiles <- function(seqnames, nmask_per_seq,
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### The "BSgenomeDataPkgSeed" class and its low-level constructor.
+### The BSgenomeDataPkgSeed and MaskedBSgenomeDataPkgSeed classes and their
+### low-level constructors.
 ###
 
-setClass(
-    "BSgenomeDataPkgSeed",
+setClass("BSgenomeDataPkgSeed",
     representation(
         Package="character",
         Title="character",
@@ -375,13 +454,47 @@ setClass(
         seqnames="character",         # a single string containing R source code
         circ_seqs="character",        # a single string containing R source code
         mseqnames="character",        # a single string containing R source code
-        nmask_per_seq="integer",      # a single integer
         PkgDetails="character",
         SrcDataFiles1="character",
         SrcDataFiles2="character",
         PkgExamples="character",
         seqfiles_prefix="character",
-        seqfiles_suffix="character",
+        seqfiles_suffix="character"
+    ),
+    prototype(
+        Author="The Bioconductor Dev Team",
+        Maintainer="Bioconductor Package Maintainer <maintainer@bioconductor.org>",
+        License="Artistic-2.0",
+        source_url="-- information not available --",
+        seqnames="NULL",              # equivalent to "character(0)"
+        circ_seqs="NULL",             # equivalent to "character(0)"
+        mseqnames="NULL",             # equivalent to "character(0)"
+        PkgDetails="",
+        SrcDataFiles1="-- information not available --",
+        SrcDataFiles2="",
+        PkgExamples="",
+        seqfiles_prefix="",
+        seqfiles_suffix=".fa"
+    )
+)   
+
+setClass("MaskedBSgenomeDataPkgSeed",
+    representation(
+        Package="character",
+        Title="character",
+        Description="character",
+        Version="character",
+        Author="character",
+        Maintainer="character",
+        RefPkgname="character",
+        License="character",
+        source_url="character",
+        organism_biocview="character",
+        nmask_per_seq="integer",      # a single integer
+        PkgDetails="character",
+        SrcDataFiles1="character",
+        SrcDataFiles2="character",
+        PkgExamples="character",
         AGAPSfiles_type="character",
         AGAPSfiles_name="character",
         AGAPSfiles_prefix="character",
@@ -398,16 +511,11 @@ setClass(
         Maintainer="Bioconductor Package Maintainer <maintainer@bioconductor.org>",
         License="Artistic-2.0",
         source_url="-- information not available --",
-        seqnames="NULL",              # equivalent to "character(0)"
-        circ_seqs="NULL",             # equivalent to "character(0)"
-        mseqnames="NULL",             # equivalent to "character(0)"
         nmask_per_seq=0L,
         PkgDetails="",
         SrcDataFiles1="-- information not available --",
         SrcDataFiles2="",
         PkgExamples="",
-        seqfiles_prefix="",
-        seqfiles_suffix=".fa",
         AGAPSfiles_type="gap",
         AGAPSfiles_name=as.character(NA),
         AGAPSfiles_prefix="",
@@ -445,7 +553,11 @@ makeS4FromList <- function(Class, x)
     do.call(new, y)
 }
 
-BSgenomeDataPkgSeed <- function(x) makeS4FromList("BSgenomeDataPkgSeed", x)
+BSgenomeDataPkgSeed <- function(x)
+    makeS4FromList("BSgenomeDataPkgSeed", x)
+
+MaskedBSgenomeDataPkgSeed <- function(x)
+    makeS4FromList("MaskedBSgenomeDataPkgSeed", x)
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -453,15 +565,19 @@ BSgenomeDataPkgSeed <- function(x) makeS4FromList("BSgenomeDataPkgSeed", x)
 ###
 
 setGeneric("forgeBSgenomeDataPkg", signature="x",
-    function(x, seqs_srcdir=".", masks_srcdir=".", destdir=".", verbose=TRUE)
+    function(x, seqs_srcdir=".", destdir=".",
+                mode=c("rda", "fa", "fa.rz"), verbose=TRUE)
         standardGeneric("forgeBSgenomeDataPkg")
 )
 
 setMethod("forgeBSgenomeDataPkg", "BSgenomeDataPkgSeed",
-    function(x, seqs_srcdir=".", masks_srcdir=".", destdir=".", verbose=TRUE)
+    function(x, seqs_srcdir=".", destdir=".",
+                mode=c("rda", "fa", "fa.rz"), verbose=TRUE)
     {
-        require(Biobase) || stop("the Biobase package is required")
-        template_path <- system.file("BSgenomeDataPkg-template", package="BSgenome")
+        require(Biobase) ||
+            stop("the Biobase package is required")
+        mode <- match.arg(mode)
+        template_path <- system.file("pkgtemplates", "BSgenome_datapkg", package="BSgenome")
         BSgenome_version <- installed.packages()['BSgenome','Version']
         symvals <- list(
             PKGTITLE=x@Title,
@@ -483,7 +599,6 @@ setMethod("forgeBSgenomeDataPkg", "BSgenomeDataPkgSeed",
             SEQNAMES=x@seqnames,
             CIRCSEQS=x@circ_seqs,
             MSEQNAMES=x@mseqnames,
-            NMASKPERSEQ=as.character(x@nmask_per_seq),
             PKGDETAILS=x@PkgDetails,
             SRCDATAFILES1=x@SrcDataFiles1,
             SRCDATAFILES2=x@SrcDataFiles2,
@@ -504,46 +619,38 @@ setMethod("forgeBSgenomeDataPkg", "BSgenomeDataPkgSeed",
         pkgdir <- file.path(destdir, x@Package)
         ## Just to avoid codetools "no visible binding" NOTEs
         .seqnames <- .mseqnames <- NULL
-        .nmask_per_seq <- 0
-        ## Sourcing this file will set the values of the '.seqnames',
-        ## '.mseqnames' and '.nmask_per_seq' variables
+        ## Sourcing this file will set the values of the '.seqnames' and
+        ## '.mseqnames' variables
         source(file.path(pkgdir, "R", "zzz.R"), local=TRUE)
-        ## Forge the "seqlengths.rda" file
         seqs_destdir <- file.path(pkgdir, "inst", "extdata")
-        forgeSeqlengthsFile(.seqnames,
-                            prefix=x@seqfiles_prefix, suffix=x@seqfiles_suffix,
-                            seqs_srcdir=seqs_srcdir,
-                            seqs_destdir=seqs_destdir,
-                            verbose=verbose)
-        ## Forge the sequence "*.rda" files
+        if (mode == "rda") {
+            ## Forge the "seqlengths.rda" file
+            forgeSeqlengthsFile(.seqnames,
+                                prefix=x@seqfiles_prefix,
+                                suffix=x@seqfiles_suffix,
+                                seqs_srcdir=seqs_srcdir,
+                                seqs_destdir=seqs_destdir,
+                                verbose=verbose)
+        }
+        ## Forge the sequence files (either "rda", "fa", or "fa.rz")
         forgeSeqFiles(.seqnames, mseqnames=.mseqnames,
-                      prefix=x@seqfiles_prefix, suffix=x@seqfiles_suffix,
+                      prefix=x@seqfiles_prefix,
+                      suffix=x@seqfiles_suffix,
                       seqs_srcdir=seqs_srcdir,
                       seqs_destdir=seqs_destdir,
+                      mode=mode,
                       verbose=verbose)
-        if (.nmask_per_seq > 0) {
-            ## Forge the "*.masks.rda" files
-            masks_destdir <- file.path(pkgdir, "inst", "extdata")
-            forgeMasksFiles(.seqnames, .nmask_per_seq,
-                            seqs_destdir=seqs_destdir,
-                            masks_srcdir=masks_srcdir, masks_destdir=masks_destdir,
-                            AGAPSfiles_type=x@AGAPSfiles_type, AGAPSfiles_name=x@AGAPSfiles_name,
-                            AGAPSfiles_prefix=x@AGAPSfiles_prefix, AGAPSfiles_suffix=x@AGAPSfiles_suffix,
-                            RMfiles_name=x@RMfiles_name, RMfiles_prefix=x@RMfiles_prefix, RMfiles_suffix=x@RMfiles_suffix,
-                            TRFfiles_name=x@TRFfiles_name, TRFfiles_prefix=x@TRFfiles_prefix, TRFfiles_suffix=x@TRFfiles_suffix,
-                            verbose=verbose)
-        }
     }
 )
 
 setMethod("forgeBSgenomeDataPkg", "list",
-    function(x, seqs_srcdir=".", masks_srcdir=".", destdir=".", verbose=TRUE)
+    function(x, seqs_srcdir=".", destdir=".",
+                mode=c("rda", "fa", "fa.rz"), verbose=TRUE)
     {
         y <- BSgenomeDataPkgSeed(x)
         forgeBSgenomeDataPkg(y,
-            seqs_srcdir=seqs_srcdir,
-            masks_srcdir=masks_srcdir,
-            destdir=destdir, verbose=verbose)
+            seqs_srcdir=seqs_srcdir, destdir=destdir,
+            mode=mode, verbose=verbose)
     }
 )
 
@@ -607,7 +714,8 @@ read.dcf2 <- function(file, ...)
 }
 
 setMethod("forgeBSgenomeDataPkg", "character",
-    function(x, seqs_srcdir=".", masks_srcdir=".", destdir=".", verbose=TRUE)
+    function(x, seqs_srcdir=".", destdir=".",
+                mode=c("rda", "fa", "fa.rz"), verbose=TRUE)
     {
         y <- .readSeedFile(x, verbose=verbose)
         y <- as.list(y)
@@ -617,13 +725,109 @@ setMethod("forgeBSgenomeDataPkg", "character",
                 stop("'seqs_srcdir' argument is missing, and the ",
                      "'seqs_srcdir' field is missing in seed file")
         }
-        if (missing(masks_srcdir) && !is.null(y[["masks_srcdir"]]))
-            masks_srcdir <- y[["masks_srcdir"]]
-        y <- y[!(names(y) %in% c("seqs_srcdir", "masks_srcdir"))]
+        y <- y[!(names(y) %in% "seqs_srcdir")]
         forgeBSgenomeDataPkg(y,
-            seqs_srcdir=seqs_srcdir,
-            masks_srcdir=masks_srcdir,
-            destdir=destdir, verbose=verbose)
+            seqs_srcdir=seqs_srcdir, destdir=destdir,
+            mode=mode, verbose=verbose)
+    }
+)
+
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### The "forgeMaskedBSgenomeDataPkg" generic and methods.
+###
+
+setGeneric("forgeMaskedBSgenomeDataPkg", signature="x",
+    function(x, masks_srcdir=".", destdir=".", verbose=TRUE)
+        standardGeneric("forgeMaskedBSgenomeDataPkg")
+)
+
+setMethod("forgeMaskedBSgenomeDataPkg", "MaskedBSgenomeDataPkgSeed",
+    function(x, masks_srcdir=".", destdir=".", verbose=TRUE)
+    {
+        require(Biobase) ||
+            stop("the Biobase package is required")
+        require(x@RefPkgname, character.only=TRUE) ||
+            stop("the ", x@RefPkgname, " package is required")
+        ref_envir <- as.environment(paste0("package:", x@RefPkgname))
+        ref_bsgenome <- get(x@RefPkgname, envir=ref_envir)
+        template_path <- system.file("pkgtemplates", "MaskedBSgenome_datapkg", package="BSgenome")
+        BSgenome_version <- installed.packages()['BSgenome','Version']
+        symvals <- list(
+            PKGTITLE=x@Title,
+            PKGDESCRIPTION=x@Description,
+            PKGVERSION=x@Version,
+            AUTHOR=x@Author,
+            MAINTAINER=x@Maintainer,
+            BSGENOMEVERSION=BSgenome_version,
+            REFPKGNAME=x@RefPkgname,
+            LIC=x@License,
+            ORGANISM=organism(ref_bsgenome),
+            SPECIES=species(ref_bsgenome),
+            PROVIDER=provider(ref_bsgenome),
+            PROVIDERVERSION=providerVersion(ref_bsgenome),
+            RELEASEDATE=releaseDate(ref_bsgenome),
+            RELEASENAME=releaseName(ref_bsgenome),
+            SOURCEURL=x@source_url,
+            ORGANISMBIOCVIEW=x@organism_biocview,
+            NMASKPERSEQ=as.character(x@nmask_per_seq),
+            PKGDETAILS=x@PkgDetails,
+            SRCDATAFILES1=x@SrcDataFiles1,
+            SRCDATAFILES2=x@SrcDataFiles2,
+            PKGEXAMPLES=x@PkgExamples
+        )
+        ## Should never happen
+        if (any(duplicated(names(symvals)))) {
+            str(symvals)
+            stop("'symvals' contains duplicated symbols")
+        }
+        ## All symvals should by single strings (non-NA)
+        is_OK <- sapply(symvals, isSingleString)
+        if (!all(is_OK)) {
+            bad_syms <- paste(names(is_OK)[!is_OK], collapse=", ")
+            stop("values for symbols ", bad_syms, " are not single strings")
+        }
+        createPackage(x@Package, destdir, template_path, symvals)
+        ## Forge the "*.masks.rda" files
+        seqs_destdir <- dirname(seqlengthsFilepath(ref_bsgenome@single_sequences))
+        pkgdir <- file.path(destdir, x@Package)
+        masks_destdir <- file.path(pkgdir, "inst", "extdata")
+        forgeMasksFiles(seqnames(ref_bsgenome), x@nmask_per_seq,
+                        seqs_destdir=seqs_destdir, mode="fa.rz",
+                        masks_srcdir=masks_srcdir, masks_destdir=masks_destdir,
+                        AGAPSfiles_type=x@AGAPSfiles_type, AGAPSfiles_name=x@AGAPSfiles_name,
+                        AGAPSfiles_prefix=x@AGAPSfiles_prefix, AGAPSfiles_suffix=x@AGAPSfiles_suffix,
+                        RMfiles_name=x@RMfiles_name, RMfiles_prefix=x@RMfiles_prefix, RMfiles_suffix=x@RMfiles_suffix,
+                        TRFfiles_name=x@TRFfiles_name, TRFfiles_prefix=x@TRFfiles_prefix, TRFfiles_suffix=x@TRFfiles_suffix,
+                        verbose=verbose)
+    }
+)
+
+setMethod("forgeMaskedBSgenomeDataPkg", "list",
+    function(x, masks_srcdir=".", destdir=".", verbose=TRUE)
+    {
+        y <- MaskedBSgenomeDataPkgSeed(x)
+        forgeMaskedBSgenomeDataPkg(y,
+            masks_srcdir=masks_srcdir, destdir=destdir,
+            verbose=verbose)
+    }
+)
+
+setMethod("forgeMaskedBSgenomeDataPkg", "character",
+    function(x, masks_srcdir=".", destdir=".", verbose=TRUE)
+    {
+        y <- .readSeedFile(x, verbose=verbose)
+        y <- as.list(y)
+        if (missing(masks_srcdir)) {
+            masks_srcdir <- y[["masks_srcdir"]]
+            if (is.null(masks_srcdir))
+                stop("'masks_srcdir' argument is missing, and the ",
+                     "'masks_srcdir' field is missing in seed file")
+        }
+        y <- y[!(names(y) %in% "masks_srcdir")]
+        forgeMaskedBSgenomeDataPkg(y,
+            masks_srcdir=masks_srcdir, destdir=destdir,
+            verbose=verbose)
     }
 )
 
